import 'package:flutter/material.dart';
import 'package:dio/dio.dart';

import 'package:flutter_google_places_web/src/search_results_tile.dart';

class FlutterGooglePlacesWeb extends StatefulWidget {
  static Map<String, String> value;
  static bool showResults = false;
  final String fieldName;
  final String apiKey;
  final String proxyURL;
  final int offset;

  ///Currently, you can use components to filter by up to 5 countries.
  ///Countries must be passed as a two character, ISO 3166-1 Alpha-2 compatible country code.
  ///For example: components=country:fr would restrict your results to places within France.
  ///Multiple countries must be passed as multiple country:XX filters, with the pipe character (|) as a separator.
  ///For example: components=country:us|country:pr|country:vi|country:gu|country:mp would restrict your results to places within the United States and its unincorporated organized territories.
  final String components;

  FlutterGooglePlacesWeb({Key key, @required this.fieldName, @required this.apiKey, this.proxyURL, this.offset, this.components});

  static FlutterGooglePlacesWebState of(BuildContext context) => context.findAncestorStateOfType<FlutterGooglePlacesWebState>();

  @override
  FlutterGooglePlacesWebState createState() => FlutterGooglePlacesWebState();
}

class FlutterGooglePlacesWebState extends State<FlutterGooglePlacesWeb> {
  final controller = TextEditingController();
  List<Address> displayedResults = [];
  String proxiedURL;
  String offsetURL;
  String componentsURL;

  Future<List<Address>> getLocationResults(String inputText) async {
    //if there is no value in the textField - close search results
    if (inputText.isEmpty) {
      setState(() {
        FlutterGooglePlacesWeb.showResults = false;
      });
    }
    String baseURL = 'https://maps.googleapis.com/maps/api/place/autocomplete/json';
    String type = 'address';
    //TODO: do session token
    String input = Uri.encodeComponent(inputText);
    if (widget.proxyURL == null) {
      proxiedURL = '$baseURL?input=$input&key=${widget.apiKey}&type=$type';
    } else {
      proxiedURL = '${widget.proxyURL}$baseURL?input=$input&key=${widget.apiKey}&type=$type';
    }
    if (widget.offset == null) {
      offsetURL = proxiedURL;
    } else {
      offsetURL = proxiedURL + '&offset=${widget.offset}';
    }
    if (widget.components == null) {
      componentsURL = offsetURL;
    } else {
      componentsURL = offsetURL + '&components=${widget.components}';
    }
    Response response = await Dio().get(componentsURL);
    var predictions = response.data['predictions'];
    if (predictions != []) {
      displayedResults.clear();
    }

    for (var i = 0; i < predictions.length; i++) {
      String name = predictions[i]['description'];
      String streetAddress = predictions[i]['structured_formatting']['main_text'];
      List<dynamic> terms = predictions[i]['terms'];
      String city = terms[terms.length - 2]['value'];
      String country = terms[terms.length - 1]['value'];
      displayedResults.add(Address(
        name: name,
        streetAddress: streetAddress,
        city: city,
        country: country,
      ));
    }
    if (displayedResults.isNotEmpty) {
      setState(() {
        FlutterGooglePlacesWeb.showResults = true;
      });
    }

    return displayedResults;
  }

  selectResult(Address clickedAddress) {
    setState(() {
      FlutterGooglePlacesWeb.showResults = false;
      controller.text = clickedAddress.name;
      FlutterGooglePlacesWeb.value['name'] = clickedAddress.name;
      FlutterGooglePlacesWeb.value['streetAddress'] = clickedAddress.streetAddress;
      FlutterGooglePlacesWeb.value['city'] = clickedAddress.city;
      FlutterGooglePlacesWeb.value['country'] = clickedAddress.country;
      print(FlutterGooglePlacesWeb.value);
    });
  }

  @override
  void initState() {
    FlutterGooglePlacesWeb.value = {};
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Flexible(
      fit: FlexFit.loose,
      child: Container(
        alignment: Alignment.center,
        child: Stack(
          children: [
            //search field
            TextFormField(
              key: widget.key,
              controller: controller,
              decoration: InputDecoration(
                hintText: 'Search',
              ),
              onChanged: (text) {
                setState(() {
                  getLocationResults(text);
                });
              },
            ),
//            FlutterGooglePlacesWeb.showResults
//                ? Container(
//                    width: MediaQuery.of(context).size.width,
//                    height: 271,
//                    child: GestureDetector(
//                      onTap: () {
//                        setState(() {
//                          FlutterGooglePlacesWeb.showResults = false;
//                        });
//                      },
//                    ),
//                  )
//                : Container(),
            FlutterGooglePlacesWeb.showResults
                ? Padding(
                    padding: EdgeInsets.only(top: 50),
                    child: Container(
                      width: MediaQuery.of(context).size.width,
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.start,
                        mainAxisSize: MainAxisSize.min,
                        children: [
                          Flexible(
                            fit: FlexFit.loose,
                            child: ListView(
                              shrinkWrap: true,
                              children: displayedResults.map((Address addressData) => SearchResultsTile(addressData: addressData, callback: selectResult, address: FlutterGooglePlacesWeb.value)).toList(),
                            ),
                          ),
                          Container(
                            height: 30,
                            child: Image.asset(
                              'packages/flutter_google_places/assets/google_white.png',
                              scale: 3,
                            ),
                          ),
                        ],
                      ),
                      decoration: BoxDecoration(
                        color: Colors.white,
                        border: Border.all(color: Colors.grey[200], width: 0.5),
                      ),
                    ),
                  )
                : Container(),
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}

class Address {
  String name;
  String streetAddress;
  String city;
  String country;
  Address({this.name, this.streetAddress, this.city, this.country});
}
